@@ -27,7 +27,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 error_reporting(E_PARSE | E_ERROR);
 if (strpos($opts='^'.implode('_', $_SERVER["argv"]).'$', "--debug") + strpos($opts, "--verbose")) { error_reporting(E_ALL); } 
 
-define("VERSION", "2.2.7");
+define("VERSION", "2.2.8");
 
 // Hard configuration and defaults
 
@@ -965,7 +965,7 @@ function nw_server_vars($include_cgi_vars=false) {
 
 	if ($include_cgi_vars && ($rq_hdrs=$GLOBALS["htreq_headers"])) foreach($rq_hdrs as $key=>$val) $nsv["HTTP_".str_replace("-", "_", $key)]=$val;
 
-	return($nsv);
+	return $nsv;
 
 }
 
@@ -1128,7 +1128,7 @@ function nw_error_page($rq_err) {
 	$err["error_label"]=$HTTP_HEADERS[$rq_err];
 	$err["error_add_message"]=$add_errmsg;
 
-	unset($add_errmsg);
+	$add_errmsg = "";
 	
 	$err["error_resource"]=($http_resource?(nw_apply_template(NW_TMPL_ERROR_RESOURCE, array("resource_name" => htmlentities($http_resource)), true)):"");
 	if ($conf[$vhost]["serveradmin"][0]) $err["error_admin"]=nw_apply_template(NW_TMPL_ERROR_ADMIN, array("admin" => $conf[$vhost]["serveradmin"][0]), true);
@@ -1762,7 +1762,7 @@ function spawn_loggers($nb_loggers) {
 set_time_limit(0);
 $pmode="master";
 
-techo("aEGiS nanoweb/".VERSION." (C) 2002-2004 by sIX / aEGiS");
+techo("aEGiS nanoweb/".VERSION." (C) 2002-2005 by sIX / aEGiS");
 
 $stats_start=time();
 
@@ -1911,8 +1911,8 @@ if ($conf["global"]["servermode"][0]!="inetd") {
 			$sbuf=$getsockopt($sock, SOL_SOCKET, SO_SNDBUF);
 			$rbuf=$getsockopt($sock, SOL_SOCKET, SO_RCVBUF);
 
-			if ($sbuf<SCK_WRITE_PACKET_SIZE) $setsockopt($sock, SOL_SOCKET, SO_SNDBUF, SCK_WRITE_PACKET_SIZE);
-			if ($rbuf<SCK_READ_PACKET_SIZE) $setsockopt($sock, SOL_SOCKET, SO_RCVBUF, SCK_READ_PACKET_SIZE);
+			if ($sbuf<(SCK_WRITE_PACKET_SIZE*32)) $setsockopt($sock, SOL_SOCKET, SO_SNDBUF, SCK_WRITE_PACKET_SIZE*32);
+			if ($rbuf<(SCK_READ_PACKET_SIZE*32)) $setsockopt($sock, SOL_SOCKET, SO_RCVBUF, SCK_READ_PACKET_SIZE*32);
 
 		}
 		
@@ -1967,9 +1967,9 @@ if ($start_daemon) {
 
 	if (!$posix_av || !$pcntl_av) errexit("posix and pcntl PHP extensions are needed for --start-daemon");
 
-	$npid=pcntl_fork();
+	$npid = pcntl_fork();
 
-	if ($npid==-1) {
+	if ($npid == -1) {
 		
 		errexit("unable to pcntl_fork()");
 
@@ -1982,15 +1982,16 @@ if ($start_daemon) {
 	posix_setsid();
 	usleep(100000);
 
-	$npid=pcntl_fork();
+	$npid = pcntl_fork();
 
-	if ($npid==-1) {
+	if ($npid == -1) {
 		
 		errexit("unable to pcntl_fork()");
 
 	} else if ($npid) {
 
 		techo("running in background");
+
 		exit(0);
 
 	}
@@ -2344,7 +2345,16 @@ while (true) {
 							$hd_key=strtoupper(strtok($s, ":"));
 							$hd_val=trim(strtok(""));
 
-							$htreq_headers[$hd_key]=$hd_val;
+							if (isset($htreq_headers[$hd_key])) {
+
+								$pri_err=400;
+								$add_errmsg="Duplicate request header '{$hd_key}'<br><br>";
+
+							} else {
+							
+								$htreq_headers[$hd_key]=$hd_val;
+
+							}
 
 						}
 
@@ -2777,21 +2787,15 @@ while (true) {
 
 						$pri_err=413;
 					
-					} else do {
+					} else {
 							
-						if ($http_continue) {
+						if ($http_continue && !$pri_err) send_response(HTTP_VERSION." ".$HTTP_HEADERS[100]."\r\n\r\n", $sck_connected);
 
-							if ($pri_err) break;
-
-							send_response(HTTP_VERSION." ".$HTTP_HEADERS[100]."\r\n", $sck_connected);
-
-						}
-						
 						$buf=$add_req;
 						if (strlen($buf)<$htreq_headers["CONTENT-LENGTH"]) $buf.=read_request($sck_connected, $dp, $pn, $htreq_headers["CONTENT-LENGTH"]-strlen($buf));
 						$htreq_content=substr($buf, 0, $htreq_headers["CONTENT-LENGTH"]);
 
-					} while (0);
+					}
 				
 				}
 				
